{% extends "base.html" %}
{% block title %}Home{% endblock %}
{% block head %}
    {{ super() }}
    <link rel="stylesheet" type="text/css" href="/static/main.css">
{% endblock %}
{% block content %}
    <h2>Home</h2>
    <div>
        Nothing to put onto this page yet...
    </div>
{% endblock %}
