.item {
    border-style: solid;
    border-color: lightgray;
    border-width: 1px;
    margin: 0px;
    margin-top: 20px;
    margin-bottom: 20px;
    display: flex;
    flex-direction: column;
}

div.test-report>*{
    margin-left: 10px;
}

h3 {
    margin-bottom: 5px;
    margin-top: 10px;
}

a.test-report {
    color: black;
    text-decoration: none;
}

div.success-bar-container {
    margin-top: 10px;
    margin-left: 0px;
    content: " ";
    background-color: lightgray;
    width: 100%;
    height: 10px;
}

div.success-bar {
    content: " ";
    background-color: #35e95f;
    height: 10px;
}

div.time-container {
    display: flex;
    flex-direction: row;
    justify-content: space-between;
}

div.time-container>div {
    margin-right: 20px;
}
