body {
    width: 100%;
    height: 100%;
    background-color: lightgray; /* #c0deeb; */
    font-size: 1.2em;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
    line-height: 1.6em;
}

.content {
    width: 60%;
    margin-right: auto;
    margin-left: auto;
    margin-top: 60px;
    margin-bottom: 60px;
    background-color: white;
    padding: 0;
    padding-bottom: 20px;
    display: flex;
    flex-direction: column;
    box-shadow: 0px 0px 10px #363e4b;
}

.content>* {
    margin-left: 30px;
    margin-right: 30px;
}

a {
    color: white;
    font-family: "Bitstream Vera Sans Mono", Monaco, "Courier New", Courier, monospace;
    padding: 3px;
    padding-left: 6px;
    padding-right: 6px;
    background-color: #ffb703;
    border-width: 0;
    border-style: solid;
    border-bottom-width: 3px;
    border-right-width: 2px;
    border-color: #fb8500;
    border-radius: 2px;
    font-size: 0.9em;
    text-decoration: none;
}

.title-container {
    margin: 0;
    padding-left: 20px;
    padding-top: 10px;
    color: white;
    font-size: 1.7em;
    background-color: #219ebc;
    border-width: 0;
    border-style: solid;
    border-bottom-width: 6px;
    border-color: #023047;
}

.title-container>p{
    margin-top: 0;
    font-size: 0.8em;
}

.title-container>h1{
    margin-bottom: 25px;
}

h2 {
    margin-top: 80px;
    border-style: solid;
    border-width: 0;
    border-bottom-width: 4px;
    border-color: #219ebc;
}

.summary {
    display: flex;
    flex-direction: column;
}

.summary>div {
    display: flex;
    flex-direction: row;
}

.summary>div>div {
    width: 50%;
}

h3.test {
    margin-top: 20px;
    border-style: solid;
    border-width: 0px;
    border-left-width: 8px;
    padding-left: 10px;
    padding-top: 5px;
    border-color: gray;
    color: gray;
}
h3.test.passing {
    border-color: #35e95f;
    color: #35e95f;
}
h3.test.error {
    border-color: red;
    color: red;
}


div.test-description {
    color: gray;
    margin-bottom: 10px;
}
