<!DOCTYPE html>
<html lang="en">
<head>
    {% block head %}
    <meta charset="UTF-8">
    <title>{% block title %}{% endblock %} - UfoTest</title>
    <link rel="stylesheet" type="text/css" href="/static/base.css">
    {% endblock %}
</head>
<body>
    <div class="header-container">
        <div class="header">
        <span class="logo">UfoTest</span>
        <div class="nav-container">
            <a class="nav-item" href="{{ config.url() }}}'">Home</a>
            <a class="nav-item" href="{{ config.url('archive') }}">Test Archive</a>
            <a class="nav-item" href="{{ config.url('builds') }}">Build Archive</a>
        </div>
    </div>
    </div>
    <div class="content">
    {% block content %}
    {% endblock %}
    </div>
</body>
</html>
