<!DOCTYPE>
<html lang="en">
    <head>
        <title>Build Report</title>
    </head>
    <body>
        <div class="content">

            <p class="greetings">
                Hello {{ name }},
            </p>

            <p>
                An ufotest build process just terminated! You are being informed, because you either initiated the
                build process by pushing to the <strong>{{ report.repository_name }}</strong> or because you own this
                repository.<br>
                The build process was started <strong>{{ report.start }}</strong> and terminated after a total of
                <strong>{{ report.duration }} minutes</strong>.
            </p>

            <p>
                Check out the results of the build process here:
                <a href="http://{{ config['context']['hostname'] }}:{{ config['context']['port'] }}/builds/{{ report.folder_name }}/report.html">Build Report</a><br>
            </p>

            <p>
                This email was automatically generated by the UfoTest CI routine.
            </p>

        </div>
    </body>
</html>
