body {
    width: 100%;
    height: 100%;
    margin: 0px;
    background-color: lightgray; /* #c0deeb; */
    font-size: 1.2em;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
    line-height: 1.6em;
}

h2 {
    margin-top: 40px;
    border-style: solid;
    border-width: 0;
    border-bottom-width: 4px;
    border-color: #219ebc;
}

div.header-container {
    width: 100%;
    margin: 0px;
    margin-bottom: 80px;
    background-color: #219ebc;
}

div.header {
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: space-between;
    margin-right: auto;
    margin-left: auto;
    width: 80%;
}

span.logo {
    font-size: 1.3em;
    font-family: 'Consolas', 'Deja Vu Sans Mono', 'Bitstream Vera Sans Mono', monospace;
    margin: 15px;
    color: white;
}

div.nav-container{
    display: flex;
    flex-direction: row;
}

a.nav-item {
    text-decoration: none;
    color: white;
    margin-left: 30px;
    border-style: solid;
    border-width: 0px;
    border-bottom-width: 3px;
    border-color: #219ebc;
}

a.nav-item:hover {
    border-color: #ffb703;
}

a.nav-item:visited {
    text-decoration: none;
}

.content {
    width: 80%;
    margin-right: auto;
    margin-left: auto;
    margin-top: 60px;
    margin-bottom: 60px;
    background-color: white;
    padding: 0;
    padding-bottom: 20px;
    display: flex;
    flex-direction: column;
    /* box-shadow: 0px 0px 10px #363e4b; */
}

.content>* {
    margin-left: 30px;
    margin-right: 30px;
}

.item {
    border-style: solid;
    border-color: lightgray;
    border-width: 1px;
    margin: 0px;
    margin-top: 20px;
    margin-bottom: 20px;
    display: flex;
    flex-direction: column;
}
